/**
 ** chksum.c - helper program to calculate simple byte checksum
 **            of provided values.
 **
 **     usage: the program takes space separated hex values via
 **            the command-line or from STDIN (NOT from both),
 **            computes a simple checksum (adds them up within
 **            the confines of an unsigned char storage space),
 **            and displays this 1 byte summed value to STDOUT.
 **            The program terminates upon exhausting all the
 **            command-line arguments, or when an EOF is
 **            encountered while processing STDIN.
 **
 **     notes: logic isn't thorough; command-line processing
 **            assumes user always does the right thing.
 **
 **            program logic assumes only 1 byte values.
 **
 **   compile: gcc -o chksum chksum.c -Wall
 **   execute: chksum de ad be ef
 **        or: echo "de ad be ef" | chksum
 **/
#include <stdio.h>
#include <stdlib.h>

typedef unsigned char uc;

int main(int argc, char **argv)
{
    //////////////////////////////////////////////////////////////////
    //
    // A few variables to help along the way
    //
    int  index         = 0;
    uc   value         = 0;
    uc   input         = 0;

    //////////////////////////////////////////////////////////////////
    //
    // Determine operating mode (command-line args (this) XOR STDIN)
    //
    if (argc          >  1)
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through each argument
        //
        for (index = 1; index < argc; index++)
        {
            //////////////////////////////////////////////////////////
            //
            // Sum each provided value to compute the checksum
            //
            value      = value + (uc) strtol(*(argv+index), NULL, 16);
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // STDIN mode
    //
    else
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through the space-separated input
        //
        do
        {
            //////////////////////////////////////////////////////////
            //
            // Grab a 1 byte hex value from STDIN
            //
            index      = fscanf(stdin, "%hhx", &input);

            //////////////////////////////////////////////////////////
            //
            // Check for error and bail out if so
            //
            if (index == 0)
            {
                fprintf(stderr, "*** ERROR: Invalid input!\n");
                exit(1);
            }

            //////////////////////////////////////////////////////////
            //
            // If not an EOF character, calculate sum
            //
            if (index != -1)
                value  = value + input;

        } while (!feof(stdin));
    }

    //////////////////////////////////////////////////////////////////
    //
    // Display the result
    //
    fprintf(stdout, "%.2hhx\n", value);

    return(0);
}
