/**
 ** endian.c - helper program to swap pairs of hex values in 
 **            order to change their endian-ness.
 **
 **     usage: the program takes pairs of space separated hex
 **            values via the command-line or from STDIN (NOT
 **            from both), swaps their order, and then displays
 **            the swapped values to STDOUT in a similarly
 **            spaces- separated fashion. The program terminates
 **            upon exhausting all the command-line arguments,
 **            or when an EOF is encountered while processing
 **            STDIN.
 **
 **     notes: logic isn't thorough; command-line processing
 **            assumes user always does the right thing.
 **
 **            program logic assumes only 1 byte values.
 **
 **   compile: gcc -o endian endian.c -Wall
 **   execute: endian ad de ef be
 **        or: echo "ad de ef be" | endian
 **/
#include <stdio.h>
#include <stdlib.h>

typedef unsigned char uc;

int main(int argc, char **argv)
{
    //////////////////////////////////////////////////////////////////
    //
    // A few variables to help along the way
    //
    int               index   = 0;
    uc                value1  = 0;
    uc                value2  = 0;

    //////////////////////////////////////////////////////////////////
    //
    // Ensure an even number of values were given
    //
    if ((argc % 2)           != 1)
    {
        fprintf(stderr, "*** ERROR: pairs of hex values must be given!\n");
        exit(1);
    }

    //////////////////////////////////////////////////////////////////
    //
    // Determine operating mode (command-line args (this) XOR STDIN)
    //
    if (argc                 >  1)
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through each argument
        //
        for(index = 1; index < argc; index+=2)
        {
            //////////////////////////////////////////////////////////
            //
            // Convert string to a base 16 number
            //
            value1            = (uc) strtol(*(argv+index),   NULL, 16);
            value2            = (uc) strtol(*(argv+index+1), NULL, 16);

            //////////////////////////////////////////////////////////
            //
            // Display the result
            //
            fprintf(stdout, "%.2hhx %.2hhx ", value2, value1);
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // STDIN mode
    //
    else
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through the space-separated input
        //
        do
        {
            //////////////////////////////////////////////////////////
            //
            // Grab a 1 byte hex value from STDIN
            //
            index             = fscanf(stdin, "%hhx", &value1);
            if (index        == -1)
                break;

            //////////////////////////////////////////////////////////
            //
            // Grab another 1 byte hex value from STDIN
            //
            index             = fscanf(stdin, "%hhx", &value2);
            if (index        == -1)
                break;

            //////////////////////////////////////////////////////////
            //
            // Check for error and bail out if so
            //
            if (index        == 0)
            {
                fprintf(stderr, "*** Error: Invalid input value!\n");
                exit(0);
            }

            fprintf(stdout, "%.2hhx %.2hhx ", value2, value1);

        } while (!feof(stdin));
    }

    //////////////////////////////////////////////////////////////////
    //
    // Newline at the end
    //
    fprintf(stdout, "\n");

    return(0);
}
