/**
 ** invert.c - helper program to convert hex numbers into their
 **            inverse.
 **
 **     usage: the program takes space separated hex values via
 **            the command-line or from STDIN (NOT from both),
 **            inverts them, and then displays the inverted
 **            value in a space separated output. The program
 **            terminates upon exhausting all the command-line
 **            arguments, or when an EOF is encountered while
 **            processing STDIN.
 **
 **     notes: logic isn't thorough; command-line processing
 **            assumes user always does the right thing.
 **
 **            program logic assumes only 1 byte values.
 **
 **   compile: gcc -o invert invert.c -Wall
 **   execute: invert de ad be ef 00 01 5a
 **        or: echo "de ad be ef 00 01 5a" | invert
 **/
#include <stdio.h>
#include <stdlib.h>

typedef unsigned char uc;

int main(int argc, char **argv)
{
    //////////////////////////////////////////////////////////////////
    //
    // A few variables to help along the way
    //
    int               index   = 0;
    uc                value   = 0;
    uc                invert  = 0;

    //////////////////////////////////////////////////////////////////
    //
    // Determine operating mode (command-line args (this) XOR STDIN)
    //
    if (argc                 >  1)
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through each argument
        //
        for(index = 1; index < argc; index ++)
        {
            //////////////////////////////////////////////////////////
            //
            // Convert string to a base 16 number
            //
            value             = (uc) strtol(*(argv+index), NULL, 16);

            //////////////////////////////////////////////////////////
            //
            // Invert the bits
            //
            invert            = ~value;

            //////////////////////////////////////////////////////////
            //
            // Display the result
            //
            fprintf(stdout, "%.2hhx ", invert);
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // STDIN mode
    //
    else
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through the space-separated input
        //
        do
        {
            //////////////////////////////////////////////////////////
            //
            // Grab a 1 byte hex value from STDIN
            //
            index             = fscanf(stdin, "%hhx", &value);

            //////////////////////////////////////////////////////////
            //
            // Check for error and bail out if so
            //
            if (index        == 0)
            {
                fprintf(stderr, "*** Error: Invalid input value!\n");
                exit(0);
            }

            //////////////////////////////////////////////////////////
            //
            // If EOF character, break out
            //
            if (index        == -1)
                break;

            //////////////////////////////////////////////////////////
            //
            // Invert the bits
            //
            invert            = ~value;

            //////////////////////////////////////////////////////////
            //
            // Display inverted value
            //
            fprintf(stdout, "%.2hhx ", invert);

        } while (!feof(stdin));
    }

    //////////////////////////////////////////////////////////////////
    //
    // Newline at the end
    //
    fprintf(stdout, "\n");

    return(0);
}
