/**
 ** search.c - helper program to search a binary file for a set
 **            sequence of hex values. It will display the
 **            offset along with the matched pattern.
 **
 **     usage: the program takes a pattern of space-separated
 **            byte values (0-9a-f), along with a half-byte
 **            "any value" regex: '.'. Source data is given
 **            via STDIN (typically via the receiving end of
 **            a pipe.
 **
 **     notes: logic isn't thorough; command-line processing
 **            assumes user always does the right thing.
 **
 **            program logic assumes only 1 byte values.
 **
 **   compile: gcc -o search search.c -Wall
 **   execute: cat file | search 'de ad be ef'
 **        or: echo "de ad be ef 00 01 5a" | search 'de ad'
 **/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned char      uc;
typedef unsigned short int usi;
typedef unsigned long int  uli;

int main(int argc, char **argv)
{
    //////////////////////////////////////////////////////////////////
    //
    // A few variables to help along the way
    //
    FILE *input                        = 0;
    int   status                       = 0;
    int   length                       = 0;
    uli   addr                         = 0;
    uli   matchaddr                    = 0;
    int   i                            = 0;
    int   j                            = 0;
    int   flag                         = 0;
    uc    upper                        = 0;
    uc    lower                        = 0;
    usi  *data                         = NULL;
    uc   *match                        = NULL;

    //////////////////////////////////////////////////////////////////
    //
    // Check for minimal number of arguments (ie PATTERN)
    //
    if (argc                          == 1)
    {
        fprintf(stderr, "Usage: %s PATTERN [FILE]\n", *(argv+0));
        exit(1);
    }

    //////////////////////////////////////////////////////////////////
    //
    // Quickly analyze pattern, get length
    //
    while (*(*(argv+1)+j)             != '\0')
    {
        if (!*(*(argv+1)+j)           == ' ')
        {
            length                     = length + 1;
        }
        j                              = j + 1;
    }

    //////////////////////////////////////////////////////////////////
    //
    // Ensure we have enough symbols to match whole bytes
    //
    if ((length % 2)                  == 1)
    {
        fprintf(stderr, "Error: Malformed PATTERN '%s'\n", *(argv+1));
        exit(1);
    }

    //////////////////////////////////////////////////////////////////
    //
    // Check for optional file argument (read from STDIN otherwise)
    //
    if (argc                          == 2)
    {
        input                          = stdin;
    }
    else
    {
        input                          = fopen(*(argv+2), "r");
        if (input                     == NULL)
        {
            fprintf(stderr, "Error: Could not open '%s' for reading\n", *(argv+2));
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // Since we operate in groups of 2 (for a whole byte), cut our
    // length value in half.
    //
    length                             = length / 2;

    //////////////////////////////////////////////////////////////////
    //
    // Allocate storage for our data and matched patterns
    //
    data                               = (usi *) malloc (sizeof(usi) * length);
    match                              = (uc *) malloc (sizeof(uc) * length);

    j                                  = 0;
    for (i = 0; i < strlen(*(argv+1)); i+=2)
    {
        flag                           = 0;
        upper                          = *(*(argv+1)+i);
        if ((upper                    >= '0') &&
            (upper                    <= '9'))
        {
            upper                      = upper - 48;
        }
        else if ((upper               >= 'a') &&
                 (upper               <= 'f'))
        {
            upper                      = upper - 87;
        }
        else if ((upper               >= 'A') &&
                 (upper               <= 'F'))
        {
            upper                      = upper - 55;
        }
        else if (upper                == '.')
        {
            flag                       = 1;
            upper                      = 0;
        }
        else
        {
            fprintf(stderr, "Error: Invalid pattern symbol '%c' (%hhu)\n",
							upper,
							upper);
            exit(1);
        }

        upper                          = upper << 4;
        upper                          = upper & 0x00F0;
        
        lower                          = *(*(argv+(1))+(i+1));
        if ((lower                    >= '0') &&
            (lower                    <= '9'))
        {
            lower                      = lower - 48;
        }
        else if ((lower               >= 'a') &&
                 (lower               <= 'f'))
        {
            lower                      = lower - 87;
        }
        else if ((lower               >= 'A') &&
                 (lower               <= 'F'))
        {
            lower                      = lower - 55;
        }
        else if (lower                == '.')
        {
            flag                       = flag | 2;
            lower                      = 0;
        }
        else
        {
            fprintf(stderr, "Error: Invalid pattern symbol '%c' (%hhu)\n",
							lower,
							lower);
            exit(1);
        }

        lower                          = lower & 0x000F;

        *(data+j)                      = 0;
        if (flag                      == 0)
            *(data+j)                  = (upper | lower) & (0x00FF);
        else if (flag                 == 1)
            *(data+j)                  = 0xF000 | upper | lower;
        else if (flag                 == 2)
            *(data+j)                  = 0x0F00 | upper | lower;
        else
            *(data+j)                  = 0xFF00 | upper | lower;

        j                              = j + 1;
        if (*(*(argv+1)+(i+2))        == 0x20)
            i                          = i + 1;

		fprintf(stdout, "*(data+%d): %.2hu\n", j, *(data+j));
    }

    i                                  = 0;
    while(!feof(input))
    {
        upper                          = fgetc(input);
        if (upper                     == *(data+i))    
        {
            *(match+i)                 = upper & 0xFF;
            i                          = i + 1;    
            flag                       = flag + 1;
        }
        else if ((*(data+i) & 0xFF00) == 0xF000)
        {
            if ((*(data+i)  & 0x000F) == (upper & 0x000F))
            {
                *(match+i)             = upper & 0xFF;
                i                      = i + 1;    
                flag                   = flag + 1;
            }
            else
            {
                i                      = 0;
                flag                   = 0;
            }
        }
        else if ((*(data+i) & 0xFF00) == 0x0F00)
        {
            if ((*(data+i)  & 0x00F0) == (upper & 0x00F0))
            {
                *(match+i)             = upper & 0xFF;
                i                      = i + 1;    
                flag                   = flag + 1;
            }
            else
            {
                i                      = 0;
                flag                   = 0;
            }
        }
        else if ((*(data+i) & 0xFF00) == 0xFF00)
        {
            *(match+i)                 = upper & 0xFF;
            i                          = i + 1;    
            flag                       = 1;
        }
        else
        {
            i                          = 0;
            flag                       = 0;
        }

        if (flag                      == 1)
        {
                matchaddr              = addr;
        }

        if (i                         == length)
        {
            fprintf(stdout, "%lx:", matchaddr-(length-1));
            for (j = 0; j < length; j++)
                fprintf(stdout, "%.2hhx ", (*(match+j) & 0x00FF));
            fprintf(stdout, "\n");
            i                          = 0;
            matchaddr                  = 0 - 1;
        }

        addr                           = addr + 1;
    }

    return(status);
}
