/**
 ** parity.c - helper program to check the parity (odd or even)
 **            of particular binary values.
 **
 **     usage: the program takes in a binary value via the
 **            the command-line or from STDIN (NOT from both),
 **            and determines its parity (as indicated by the
 **            first argument on the command-line).
 **
 **            program will output a "true" or "false" message
 **            to indicate if it matches or does not match the
 **            specified parity (given as "odd" or "even" as 
 **            the first argument on the command-line).
 **
 **     notes: logic isn't thorough; command-line processing
 **            assumes user always does the right thing.
 **
 **            program logic does not assume any maximum lengths.
 **
 **   compile: gcc -o parity parity.c -Wall
 **   execute: parity odd 0e
 **        or: echo "0e" | parity odd
 **/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  EVEN    0
#define  ODD    1

typedef unsigned char      uc;
typedef   signed short int ssi;

int main(int argc, char **argv)
{
    //////////////////////////////////////////////////////////////////
    //
    // A few variables to help along the way
    //
    int               index                = 0;
    uc                parity               = EVEN;
    uc                value                = 0;
    uc                bit                  = 0;

    //////////////////////////////////////////////////////////////////
    //
    // Not enough arguments (or too many) were given
    //
    if ((argc                             <  2) ||
        (argc                             >  3))
    {
        fprintf(stderr, "Error: invalid argument count!\n");
        fprintf(stderr, "Usage: %s odd|even [BINARY]\n", *(argv+0));
        exit(1);
    }

    //////////////////////////////////////////////////////////////////
    //
    // Determine parity mode (even, odd, error)
    //
    if (strcasecmp(*(argv+1), "even")     == 0)
    {
        parity                             = EVEN;
    }
    else if (strcasecmp(*(argv+1), "odd") == 0) 
    {
        parity                             = ODD;
    }
    else
    {
        fprintf(stderr, "Error: invalid parity type '%s'\n", *(argv+1));
        exit(1);
    }

    //////////////////////////////////////////////////////////////////
    //
    // Determine operating mode (command-line args (this) XOR STDIN)
    //
    if (argc                              == 3)
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through each bit
        //
        while (*(*(argv+2)+index)         != '\0')
        {
            //////////////////////////////////////////////////////////
            //
            // Check for a '1' or a '0'
            //
            if (*(*(argv+2)+index)        == '1')
                value                      = value + 1;

			index                          = index + 1;
        }
    }

    //////////////////////////////////////////////////////////////////
    //
    // STDIN mode
    //
    else if (argc                         == 2)
    {
        //////////////////////////////////////////////////////////////
        //
        // Iterate through each bit of the input
        //
        do
        {
            //////////////////////////////////////////////////////////
            //
            // Grab a 1 byte hex value from STDIN
            //
            index                          = fscanf(stdin, "%c", &bit);

            //////////////////////////////////////////////////////////
            //
            // If EOF character, break out
            //
            if ((index                    == -1) ||
				(bit                      == '\n'))
			{
                break;
			}

            //////////////////////////////////////////////////////////
            //
            // Check for invalid input and bail out if so
            //
            if ((bit                      != '0') &&
				(bit                      != '1'))
            {
                fprintf(stderr, "*** Error: Invalid input value '%c'!\n", bit);
                exit(0);
            }

            //////////////////////////////////////////////////////////
            //
            // Iterate through each bit
            //
            if (bit                       == '1')
            {
                //////////////////////////////////////////////////////
                //
                // Check for a '1' or a '0'
                //
                if (bit                   == '1')
                    value                  = value + 1;
            }

        } while (!feof(stdin));
    }

    //////////////////////////////////////////////////////////////////
    //
    // Verify and display the parity
    //
    if ((value % 2)                       == parity)
        fprintf(stderr, "true");
    else
        fprintf(stderr, "false");

    //////////////////////////////////////////////////////////////////
    //
    // Newline at the end
    //
    fprintf(stdout, "\n");

    return(0);
}
